<?php

namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

use Zeedhi\Framework\DataSource\DataSet;

class MultiOrder extends \Zeedhi\Framework\Controller\Simple {
    
    const ERROR_MESSAGE     = 'E';
    const GENERAL_MESSAGE   = 'A';
    
    protected $util;
    protected $orderController;
    protected $printMode;
    protected $requestTimeout;
    protected $orderService;
    protected $multiOrderService;
    protected $systemPath;
    
    public function __construct(\Util\Util $util, 
                                \Controller\Order $orderController,
                                \Service\Order $orderService,
                                \Service\MultiOrder $multiOrderService,
                                \Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager) {
        $this->util              = $util;
        $this->orderController   = $orderController;
        $this->printMode         = $instanceManager->getParameter('PRINT_MODE');
        $this->requestTimeout    = $instanceManager->getParameter('REQUEST_TIMEOUT');
        $this->systemPath        = $instanceManager->getParameter('SYSTEM_PATH');
        $this->orderService      = $orderService;
        $this->multiOrderService = $multiOrderService;
    }
    
    public function saveOrders(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $timeout = $this->requestTimeout;
        
        try{
            if(!isset($input)) 
                throw new \Exception('Não foram recebidos parâmetros');
        }catch(\Exception $e){
            $message = 'Erro ao inserir comanda delivery. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            return;
        }
        
        $cdfilial      = $input['subsidiaryCode'];
        $cdsenhaped    = $input['pagerId'];
        $dsconsumidor  = $input['consumerId'];
        $stores        = $input['stores'];
        $cdcliente     = $input['clientCode'];
        $cdconsumidor  = $input['consumerCode'];
        $driveThru     = $input['driveThru'];
        $idorgcmdvenda = $input['orderType'];
        $nrinscrcons   = $input['orderCPF'];
        $closeByApi    = 'S';
        
        if(!empty($cdcliente) && !empty($cdconsumidor))
            $this->multiOrderService->validateConsumer($cdcliente, $cdconsumidor);
        else
            $cdcliente = $this->util->getDefaultClient($cdfilial);
        
        $ordersSeq = $this->generateOrdersSeq($cdfilial, $stores, $response);
        
        $validatedOrders = $this->validateTaaSaleId($stores);
        if($validatedOrders['error'] === true){
            $response->addMessage(new Message($validatedOrders['error_message'], self::GENERAL_MESSAGE));
            $response->addDataSet(
                new DataSet(
                    '', 
                    array($validatedOrders['error_orders'])
                )
            );
            return;
        }
        
        $now = strtotime("now");
        do{
            $this->util->beginTransaction();
            try{
                $savedOrders = $sanitazedOrders = array();
                foreach($stores as $store){
                    $orderDetail = $this->saveOrder($ordersSeq[$store['storeCode']], $store, $cdfilial, $cdsenhaped, $dsconsumidor, $cdcliente, $cdconsumidor, $driveThru, $idorgcmdvenda, $nrinscrcons, $closeByApi);
                    array_push($savedOrders, $orderDetail);
                    array_push($sanitazedOrders,array(
                        'storeCode' => $orderDetail['cdloja'],
                        'saleCode'  => $orderDetail['nrcomanda'],
                        'taaSaleId' => $orderDetail['nrcomandaext']
                    ));
                }
                $elapsedTime = strtotime("now");
                if(($elapsedTime - $timeout) >= $now)
                    throw new \Exception("Tempo limite excedido.");
                
                $this->util->commit();
                foreach($savedOrders as $orderDetail){
                    $nrcomanda  = $orderDetail['nrcomanda'];
                    $cdloja     = $orderDetail['cdloja'];
                    $items      = $orderDetail['items'];
                    $cdvendedor = $orderDetail['cdvendedor'];
                    $nrmesa     = $orderDetail['nrmesa'];
                    $toGo       = $orderDetail['idretballoja'];
                    
                    if(!empty($dsconsumidor) && !empty($cdsenhaped)) $cdsenhaped.=" - $dsconsumidor";
                    $printItems = $this->orderController->formatItemsToPrint($items);
                    if($toGo == 'S')
                        $toGoString = 'PEDIDO PARA RETIRADA NA LOJA';
                    else
                        $toGoString = 'PEDIDO PARA ENTREGA DELIVERY';
                    $this->orderController->printOrder($cdfilial, $nrcomanda, $cdloja, $printItems, $cdvendedor, $nrmesa, $cdsenhaped, $toGoString, true);
                }
                $response->addDataSet(
                    new DataSet(
                        '', 
                        $sanitazedOrders
                    )
                );
            }catch(\Exception $e){
                $this->util->rollBack();
                $message = 'Erro ao inserir comanda delivery. '.$e->getMessage();
                $elapsedTime = strtotime("now");
                if(($elapsedTime - $timeout) >= $now){
                    $response->addMessage(new Message($message, self::ERROR_MESSAGE));
                }
                continue;
            }
            
        break;
        }
        while(($elapsedTime - $timeout) <= $now);
        
    }
    
    private function generateOrdersSeq($cdfilial, $stores, $response){
        $ordersSeq = array();
        $this->util->beginTransaction();
        try{
            foreach($stores as $store){
                $ordersSeq[$store['storeCode']] = array(
                    'NRVENDAREST' => $this->orderService->generateVendarestSeq($cdfilial),
                    'NRCOMANDA'   => $this->orderService->generateComandavenSeq($cdfilial)
                );
            }
            $this->util->commit();
            return $ordersSeq;
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao gerar novo codigo. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    }
    
    private function validateTaaSaleId($stores){
        $data = array(
            'error' => false,
            'error_message' => 'Os pedidos abaixo já foram inseridos anteriormente na base de dados.',
            'error_orders' => array()
        );
        foreach($stores as $store){
            $order = $store['order'];
            $taaSaleId = isset($order['taaSaleId']) ? $order['taaSaleId'] : null;
            if(!empty($taaSaleId)){
                $orderById = $this->orderService->getOrderById($taaSaleId);
                if(!empty($orderById)){
                    $data['error'] = true;
                    if($this->util->databaseIsOracle()) $format = 'Y-m-d H:i:s';
                    else $format = 'Y-m-d H:i:s.u';
                    $orderTime = \DateTime::createFromFormat($format, $orderById['DTHRABERMESA']);
                    $orderTime = $orderTime->format($format);
                    $errorOrderData = array(
                        'taaSaleId' => $taaSaleId,
                        'createdAt' => $orderTime
                    );
                    array_push($data['error_orders'], $errorOrderData);
                }
            }
        }
        return $data;
    }
    
    private function saveOrder($ordersSeq, $store, $cdfilial, $cdsenhaped, $dsconsumidor, $cdcliente, $cdconsumidor, $driveThru, $idorgcmdvenda, $nrinscrcons, $closeByApi){
        $cdloja = $store['storeCode'];
        $order  = $store['order'];
        
        $items            = $order['items'];
        $taaSaleId        = isset($order['taaSaleId']) ? $order['taaSaleId'] : null;
        $orderDescription = $order['orderDescription'];
        $cdcaixa          = $order['posCode'];
        $orderDiscount    = $orderDescription['orderDiscount'];
        $productsValue    = $orderDescription['productsValue'];
        $totalValue       = $orderDescription['totalValue'];
        
        $itemsInfo = $this->multiOrderService->separateItems($items);
        
        $realItemsQuant = $itemsInfo['itemsQuant'];
        $minOrderValue  = $itemsInfo['minOrderValue'];
        $items          = $itemsInfo['items'];
        
        $this->multiOrderService->validateOrder($minOrderValue, $productsValue, $orderDiscount, $totalValue);
        
        $items = $this->treatItems($cdfilial, $items, $orderDiscount, $productsValue);
        
        return $this->openOrder($cdfilial, $cdloja, $cdcaixa, $items, $ordersSeq, $cdcliente, $cdconsumidor, $dsconsumidor, $orderDescription, $cdsenhaped, $driveThru, $idorgcmdvenda, $nrinscrcons, $closeByApi, $taaSaleId);
    }
    
    private function treatItems($cdfilial, $items, $orderDiscount, $productsValue){
        $items = $this->orderController->recalculateComboDiscount($cdfilial, $items);
        return $this->multiOrderService->shareDiscount($items, $orderDiscount, $orderDiscount, $productsValue, $productsValue, $orderDiscount);
    }
    
    private function openOrder($cdfilial, $cdloja, $cdcaixa, $items, $orderSeq, $cdcliente, $cdconsumidor, $dsconsumidor, $orderDescription, $cdsenhaped, $driveThru, $idorgcmdvenda, $nrinscrcons, $closeByApi, $taaSaleId){
        
        $driveThruData = array();
        if(!empty($driveThru)) 
            $driveThruData = $this->getDriveThruData();
        
        $idretballoja = $this->orderController->isToGo($idorgcmdvenda);
        $idimpcupexp  = $this->multiOrderService->setIdimpcupexp($idorgcmdvenda, $idretballoja, $this->printMode);
        
        $nrvendarest = $orderSeq['NRVENDAREST'];
        $nrcomanda   = $orderSeq['NRCOMANDA'];
        
        $standardSeller = $this->getSellerInfo($cdfilial);
        
        $cdvendedor  = $standardSeller['CDVENDEDOR'];
        $cdoperador  = $standardSeller['CDOPERADOR'];
        $nrmesa      = $standardSeller['NRMESA'];
        
        $idstcomanda = $this->getIdcontroprod($cdfilial, $cdloja);
        
        $this->orderController->insertVendaRest($cdfilial, $nrvendarest, $cdloja, $cdvendedor, $cdoperador, $cdcliente, $cdconsumidor, $nrmesa);
        $this->multiOrderService->insertComandaVen($cdfilial, $nrvendarest, $nrcomanda, $cdloja, $idstcomanda, $nrcomanda, $idorgcmdvenda, $idretballoja, $taaSaleId, $idimpcupexp, $dsconsumidor, $closeByApi);
        $this->saveItems($items, $cdfilial, $nrvendarest, $nrcomanda, $idstcomanda, $cdcaixa, $cdloja, $cdvendedor, $nrinscrcons, $cdsenhaped, $idorgcmdvenda, $driveThruData);
        return array(
            'cdloja'       => $cdloja,
            'nrcomanda'    => $nrcomanda,
            'cdvendedor'   => $cdvendedor,
            'nrmesa'       => $nrmesa,
            'items'        => $items,
            'idretballoja' => $idretballoja,
            'nrcomandaext' => $taaSaleId
        );        
    }
    
    private function getSellerInfo($cdfilial){
        $sellerInfo = $this->orderService->getStandardSeller($cdfilial);
        if($sellerInfo['error'] === true)
            throw new \Exception($sellerInfo['message']);
        else return $sellerInfo['data'];
    }
    
    private function getIdcontroprod($cdfilial, $cdloja){
        $idstcomanda = '3';
        $idcontroprod = $this->util->getIdcontroprod($cdloja, $cdfilial);
        if($idcontroprod === 'S')
            $idstcomanda = '1';
        return $idstcomanda;
    }
    
    private function saveItems($items, $cdfilial, $nrvendarest, $nrcomanda, $idcontroprod, $cdcaixa, $cdloja, $cdvendedor, $nrinscrcons, $cdsenhaped, $idorgcmdvenda, $driveThruData){
        $nrprodcomven = $this->orderService->getQuantItens($cdfilial, $nrvendarest, $nrcomanda);
        $this->multiOrderService->saveItems($items, $cdfilial, $nrvendarest, $nrcomanda, $idcontroprod, $cdcaixa, $cdloja, $cdvendedor, $nrinscrcons, $cdsenhaped, $idorgcmdvenda, $nrprodcomven, $driveThruData);
    }
    
    private function getDriveThruData(){
        $driveThruData = array();
        $driveThruData = $this->orderController->getDriveThruData();
        if($driveThruData['obsGroupCode'] === null) $driveThruData = array();
    }
    
    public function savePayment(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $timeout = $this->requestTimeout;
        
        try{
            if(!isset($input)) 
                throw new \Exception('Não foram recebidos parâmetros');
        }catch(\Exception $e){
            $message = 'Erro ao salvar pagamentos. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
        
        $cdfilial     = $input['subsidiaryCode'];
        $cdcliente    = $input['clientCode'];
        $cdconsumidor = $input['consumerCode'];
        $stores       = $input['stores'];
        $payments     = $input['payments'];
        $giftCards    = $input['giftCards'];
        $cashBack     = $input['cashBack'];
        
        $ordersValue   = $this->multiOrderService->getOrdersValue($cdfilial, $stores);
        $paymentsValue = $this->multiOrderService->totalPayments($payments);
        
        
        $giftCard = array();
        if(!empty($giftCards))
            $giftCard = $this->treatGiftCards($giftCards, $cdfilial);
            
        $this->util->beginTransaction();
        try{
            $totalGiftCard = $this->multiOrderService->totalGiftCard($giftCards);
            
            $this->validatePayment($ordersValue, $paymentsValue);        
            $this->multiOrderService->validateTotalPaid($paymentsValue, $totalGiftCard, $cashBack, $ordersValue);
            
            foreach($stores as $orderInfo){
                $cdloja     = $orderInfo['storeCode'];
                $nrcomanda  = $orderInfo['saleCode'];
                
                $order = $this->util->getNrvendarest($cdfilial, $nrcomanda);
                $nrvendarest = $order['NRVENDAREST'];
                
                $orderPaymentValue = $this->util->getPaymentValue($cdfilial, $nrvendarest);
                if(!empty($orderPaymentValue))
                    throw new \Exception("O pedido $nrcomanda já foi pago!");
                
                $itemsValue = $this->orderService->getItemsValue($cdfilial, $nrvendarest, $nrcomanda);
                $remaingValue = $itemsValue;
                $cdcaixa = $this->getCdcaixa($cdfilial, $nrcomanda, $nrvendarest);
                            
                if(!empty($payments)){
                    $treatedPayments = $this->multiOrderService->treatPayments($payments, $itemsValue);
                    $payments      = $treatedPayments['payments'];
                    $remaingValue  = $treatedPayments['remaingValue'];
                    $orderPayments = $treatedPayments['orderPayments'];
                    $this->orderController->savePayments($orderPayments, $cdfilial, $nrvendarest, $cdcliente, $cdconsumidor, 0);
                }
                
                if($remaingValue == 0)
                    continue;
                
                if(!empty($giftCard)){
                    $dtabercaix = $this->util->getPosDate($cdfilial, $cdcaixa);
                    $treatedGiftCard = $this->multiOrderService->treatGiftCard($giftCard, $cdfilial, $remaingValue);
                    $giftCard     = $treatedGiftCard['giftCard'];
                    $remaingValue = $treatedGiftCard['remaingValue'];
                    $paymentValue = $treatedGiftCard['paymentValue'];
                    $this->multiOrderService->saveGiftCard($giftCard, $cdfilial, $nrvendarest, $cdcaixa, $dtabercaix, $paymentValue);
                    if(empty($cdconsumidor))
                        $this->orderService->updateConsumerVendarest($giftCard['clientCode'], $giftCard['consumerCode'], $nrvendarest, $cdfilial);
                }
                
                if($remaingValue == 0)
                    continue;
                    
                if(!empty($cashBack)){
                    $this->orderController->validateCashBack($cdcliente, $cdconsumidor, $cashBack);
                    $this->validateCashBackPayment($remaingValue);
                    $cashBack = $this->multiOrderService->treatCashBack($itemsValue, $remaingValue, $cdfilial, $nrcomanda, $nrvendarest, $cdcliente, $cdconsumidor, $cashBack);
                }
            }
            $this->util->commit();
            $response->addMessage(new Message("Pagamento efetuado com sucesso.", self::GENERAL_MESSAGE));
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao inserir comanda delivery. '.$e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
    
    }
    
    private function validatePayment($ordersValue, $paymentsValue){
        if($paymentsValue > $ordersValue)
            throw new \Exception("Pagamento maior que a soma dos itens!");
    }
    
    private function validateCashBackPayment($remaingValue){
        if($remaingValue == 0)
            throw new \Exception("Não é possível utilizar cashback nesta compra, valor de pedido já suprido por outras formas de pagamento");
    }
    
    private function getCdcaixa($cdfilial, $nrcomanda, $nrvendarest){
        $caixa = $this->util->getCDCAIXA($cdfilial, $nrcomanda, $nrvendarest);
        return $caixa['CDCAIXACOLETOR'];
    }
    
    private function treatGiftCards($giftCards, $cdfilial){
        $giftCard = array();
        if(count($giftCards) > 1){
            $this->orderController->validateConsumerType($giftCards);
            $giftCard = $this->multiOrderService->mergeGiftCards($giftCards, $cdfilial);
        }else{
            $giftCard = $giftCards[0];
        }
        return $giftCard;
    }
    
    public function closeOrders(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $timeout = $this->requestTimeout;
        
        try{
            if(!isset($input)) 
                throw new \Exception('Não foram recebidos parâmetros');
        }catch(\Exception $e){
            $message = 'Erro ao fechar comandas. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
        
        $cdfilial = $input['subsidiaryCode'];
        $orders = $input['orders'];
        $salesData = array();
        $closedSales = array();
        foreach($orders as $orderData){
            try{
                $this->util->beginTransaction();
                $saleData = array();
                $nrcomanda = $orderData['saleCode'];
                $order = $this->orderService->getOrder($cdfilial, $nrcomanda);
                
                $cdloja = $order['CDLOJA'];
                $cdcliente = $order['CDCLIENTE'];
                $cdconsumidor = $order['CDCONSUMIDOR'];
                
                if(empty($order)) 
                    throw new \Exception("Comanda não encontrada! Verifique os dados de entrada.");
                
                $nrvendarest = $order['NRVENDAREST'];
                $xml = '';
                $status = 'N';
                $message = '';
                $dtentrvenda = null;
                
                if($order['IDSTCOMANDA'] === 'X' || $order['IDSTCOMANDA'] === 'P'){
                    $cdcaixa = $order['CDCAIXA'];
                    $saleDetails = $this->util->getSaleDetails($cdfilial, $nrcomanda, $nrvendarest, $this->systemPath, $cdcaixa);
                    $nrseqvenda  = $saleDetails['NRSEQVENDA'];
                    $cdcaixa     = $saleDetails['CDCAIXA'];
                    $status      = $saleDetails['IDSTATUSNFCE'];
                    $dtentrvenda = $saleDetails['DTENTRVENDA'];
                    $xml         = $saleDetails['DSARQXMLNFCE'];
                    $xml         = str_replace('"',"'", $xml);
                }else{
                    $payments = $this->orderService->getOrderPayments($cdfilial, $nrvendarest);
                    $paymentsValue = 0;
                    if(!empty($payments)) {
                        foreach ($payments as $payment) {
                            $paymentsValue = bcadd($payment['VRMOVIVEND'], $paymentsValue, 2);
                        }
                    }
                    
                    $cashBackValue = $this->orderService->getOrderCashback($cdfilial, $nrvendarest, $nrcomanda);
                    $items = $this->orderService->getOrderItems($nrvendarest, $nrcomanda, $cdfilial);
                    $productsValue = 0;
                    foreach ($items as $item) {
                        $productsValue = $this->util->add($productsValue, $item['TOTAL']);
                    }
                    $totalPayment = $this->util->add($cashBackValue['VRDESCFID'], $paymentsValue);
                    $totalOrderValue = $this->util->add($productsValue, $order['VRACRCOMANDA']);
                    if($totalPayment < $totalOrderValue){
                        $remaingValue = $this->util->subtract($totalOrderValue, $totalPayment);
                        throw new \Exception("Ainda devem ser pago(s) R$$remaingValue para que a comanda seja fechada.");
                    } 
                    
                    $fechamento = $this->orderService->closeOrder($order, $cdfilial, $nrcomanda);
                    $message = $fechamento['message'];
        
                    if(isset($fechamento['mensagemNfce'])) $fechamento['message'] .= $fechamento['mensagemNfce'];
        
                    if($fechamento['error'] === false){
                        $status = isset($fechamento['IDSTATUSNFCE']) ? $fechamento['IDSTATUSNFCE'] : 'S';
                        if($status === 'R')
                            throw new \Exception($fechamento['message']);
                        $nrseqvenda = $fechamento['NRSEQVENDA'];
                        $cdcaixa = $fechamento['CDCAIXA'];
                        $saleStatus = $this->util->getSaleStatus($cdfilial, $cdcaixa, $nrseqvenda);
                        $closedSale = array(
                            'CDFILIAL' => $cdfilial,
                            'NRSEQVENDA' => $nrseqvenda,
                            'CDCAIXA' => $cdcaixa,
                            'NRCOMANDA' => $nrcomanda,
                            'NRVENDAREST' => $nrvendarest,
                            'CDOPERADOR' => $saleStatus['CDOPERADOR'],
                            'CDSUPERVISOR' => $saleStatus['CDSUPERVISOR']
                        );
                        array_push($closedSales, $closedSale);
                        $xml = $this->util->getSaleXml($cdfilial, $cdcaixa, $nrcomanda);
                        $dtentrvenda = $this->util->getSaleDate($cdfilial, $cdcaixa, $nrcomanda);
                        if($status == 'P' && empty($xml)){
                            $xml = isset($fechamento['XMLENVIO']) ? $fechamento['XMLENVIO'] : '';   
                        }
                        $xml = str_replace('"',"'", $xml);
                        
                    }else{
                        throw new \Exception($fechamento['message']);
                    }
                }
                
                $storeName = $this->util->getStoreName($cdfilial, $cdcaixa, $cdloja);
                
                $fidelityPoints = 0;
                $personalCredit = 0;
                
                if(!empty($cdconsumidor)){
                    $personalCredit = $this->util->getConsumerBalance($cdcliente, $cdconsumidor, '001');
                    $fidelityPoints = $this->util->getConsumerFidelity($cdcliente, $cdconsumidor);
                    if(!empty($fidelityPoints)) $fidelityPoints = $fidelityPoints['value'];
                }
                
                $saleData = array(
                    'storeCore'      => $cdloja,
                    'saleCode'       => $nrcomanda,
                    'message'        => $message,
                    'consumerName'   => $order['NMCONSUMIDOR'],
                    'saleDate'       => $dtentrvenda,
                    'fidelityPoints' => $fidelityPoints,
                    'personalCredit' => $personalCredit,
                    'saleSeq'        => $nrseqvenda,
                    'posCode'        => $cdcaixa,
                    'posName'        => $storeName['NMCAIXA'],
                    'subsidiaryCode' => $cdfilial,
                    'subsidiaryName' => $storeName['NMFILIAL'],
                    'storeCode'      => $cdloja,
                    'storeName'      => $storeName['NMLOJA'],
                    'status'         => $status,
                    'pagerId'        => $order['CDSENHAPED'],
                    'xml'            => $xml
                );
                
                array_push($salesData, $saleData);
                $this->util->commit();
            }catch(\Exception $e){
                $this->util->rollBack();
                $salesCancelError = $this->cancelSales($closedSales);
                $message = 'Erro ao fechar comandas. '.$e->getMessage(). $salesCancelError;
                $response->addMessage(new Message($message, self::ERROR_MESSAGE));
            }
        }
        $response->addDataSet(
            new DataSet(
                '',
                $salesData
            )
        );
    }
    
    private function cancelSales($closedSales){
        $cancelError = array();
        try{
            foreach($closedSales as $sale){
                $this->util->beginTransaction();
                $cdfilial    = $sale['CDFILIAL'];
                $nrvendarest = $sale['NRVENDAREST'];
                $nrcomanda   = $sale['NRCOMANDA'];
                $cdcaixa     = $sale['CDCAIXA'];
                $nrseqvenda  = $sale['NRSEQVENDA'];
                $canceledItems = $this->util->cancelProducts($cdfilial, $nrvendarest, $nrcomanda);
                foreach ($canceledItems as $canceledItem) {
                    $kdsSeq = $this->orderService->generateKDSSeq('KDSOPERACAOTEMP');
                    $dsOperacao = array(
                        'operation' => 'cancelItem',
                        'params' => array(
                            'CDFILIAL'     => $cdfilial,
                            'NRVENDAREST'  => $nrvendarest,
                            'NRCOMANDA'    => $nrcomanda,
                            'NRPRODCOMVEN' => $canceledItem['NRPRODCOMVEN']
                        )
                    );
                    $this->util->insertKDSTEMP($kdsSeq, $dsOperacao);
                }
                $cdoperador   = $sale['CDOPERADOR'];
                $cdsupervisor = $sale['CDSUPERVISOR'];
                $result = $this->orderService->cancelOrder($cdfilial, $cdcaixa, $nrseqvenda, $cdoperador, $cdsupervisor);
                if($result['error']){
                    $error = "Erro ao cancelar venda saleCode(NRCOMANDA): $nrcomanda saleSeq(NRSEQVENDA): $nrseqvenda caixa: $cdcaixa Motivo: ".$result['message'];
                    array_push($cancelError, $error);
                }
                $this->util->commit();
            }
            if(!empty($cancelError))
                throw new \Exception();
            else
                return "Todas as vendas foram canceladas com sucesso.";
        }catch(\Exception $e){
            $this->util->rollBack();
            return "Vendas com erro de cancelamento: ".implode(" /r/n ",$cancelError);
        }
    }
    
    public function cancelOrders(Request $request, Response $response){
        $inputJSON = file_get_contents('php://input');
        $input = json_decode($inputJSON, true);
        $cancelOrderErrors = array();
        $cancelOrderData = array();
        try{
            $cdfilial = $input['subsidiaryCode'];
            $orders = $input['orders'];
            
            foreach($orders as $order){
                $this->util->beginTransaction();
                $nrcomanda = $order['saleCode'];
                $cdcaixa = $order['posCode'];
                $nrseqvenda = $order['saleSeq'];
                $nrvendarest = $this->util->getNrvendarest($cdfilial, $nrcomanda);
                $nrvendarest = $nrvendarest['NRVENDAREST'];
                if(!$nrvendarest){
                    array_push($cancelOrderErrors, array(
                        'error' => "Não foi posssível encontrar o pedido.",
                        'saleCode' => $nrcomanda,
                        'posCode' => $cdcaixa,
                        'saleSeq' => $nrseqvenda
                    ));
                }else{
                    $saleStatus = $this->util->getSaleStatus($cdfilial, $cdcaixa, $nrseqvenda);
                
                    $canceledItems = $this->util->cancelProducts($cdfilial,$nrvendarest,$nrcomanda);
                    foreach ($canceledItems as $canceledItem) {
                        $kdsSeq = $this->orderService->generateKDSSeq('KDSOPERACAOTEMP');
                        $dsOperacao = array(
                            'operation' => 'cancelItem',
                            'params' => array(
                                'CDFILIAL'     => $cdfilial,
                                'NRVENDAREST'  => $nrvendarest,
                                'NRCOMANDA'    => $nrcomanda,
                                'NRPRODCOMVEN' => $canceledItem['NRPRODCOMVEN']
                            )
                        );
                        $this->util->insertKDSTEMP($kdsSeq, $dsOperacao);
                    }
                    if(!empty($saleStatus) && isset($saleStatus)){
                        $saleValue = $this->util->removeDecimal($saleStatus['VRTOTVENDA']);
                        if($saleStatus['IDSITUVENDA'] == 'O'){
                            $cdoperador = $saleStatus['CDOPERADOR'];
                            $idstatusnfce = $saleStatus['IDSTATUSNFCE'];
                            $cdsupervisor = $saleStatus['CDSUPERVISOR'];
                            $result = $this->orderService->cancelOrder($cdfilial, $cdcaixa, $nrseqvenda, $cdoperador, $cdsupervisor);
                            if($result['error']){
                                array_push($cancelOrderErrors, array(
                                    'error' => $result['message'],
                                    'saleCode' => $nrcomanda,
                                    'posCode' => $cdcaixa,
                                    'saleSeq' => $nrseqvenda,
                                    'value' => $saleValue
                                ));
                            } else{
                                array_push($cancelOrderData, array(
                                    'saleCode' => $nrcomanda,
                                    'posCode' => $cdcaixa,
                                    'saleSeq' => $nrseqvenda,
                                    'value' => $saleValue
                                ));
                            }
                        } else if($saleStatus['IDSITUVENDA'] == 'C'){
                            array_push($cancelOrderErrors, array(
                                'error' => "A venda $nrseqvenda já foi cancelada.",
                                'saleCode' => $nrcomanda,
                                'posCode' => $cdcaixa,
                                'saleSeq' => $nrseqvenda,
                                'value'   => $saleValue
                            ));
                        } else if($saleStatus['IDSITUVENDA'] == 'P'){
                            array_push($cancelOrderErrors, array(
                                'error' => "A venda $nrseqvenda se encontra pendente.",
                                'saleCode' => $nrcomanda,
                                'posCode' => $cdcaixa,
                                'saleSeq' => $nrseqvenda,
                                'value' => $saleValue
                            ));
                        }
                    } else {
                        $this->deleteOrder($nrvendarest, $cdfilial, $nrcomanda);
                        array_push($cancelOrderData, array(
                            'saleCode' => $nrcomanda,
                            'posCode' => $cdcaixa,
                            'saleSeq' => null
                        ));
                    }
                    
                    $this->util->commit();
                }
            }
        }catch(\Exception $e){
            $this->util->rollBack();
            $message = 'Erro ao cancelar comandas/vendas. ' . $e->getMessage();
            $response->addMessage(new Message($message, self::ERROR_MESSAGE));
        }
        
        $response->addDataSet(
            new DataSet(
                '',
                array(
                    'canceledOrders' => $cancelOrderData,
                    'canceledOrdersErrors' => $cancelOrderErrors
                )
            )
        );
    }
    
    private function deleteOrder($nrvendarest, $cdfilial, $nrcomanda){
        $this->util->deleteItcomandaven($nrvendarest, $cdfilial, $nrcomanda);
        $this->util->deleteComandaven($nrvendarest, $cdfilial, $nrcomanda);
        $this->util->deleteMovCaixaDlv($nrvendarest, $cdfilial);
        $this->util->deleteVendarest($nrvendarest, $cdfilial);
    }
}